@@ -30,6 +30,7 @@
 #include <mailutils/registrar.h>
 #include <mailutils/nls.h>
 #include <mailutils/cfg.h>
+#include <mailutils/cstr.h>
 #include "muaux.h"
 
 /* This file implements mandatory locking for pop3d and imap4d.  Mandatory
@@ -258,8 +259,9 @@ manlock_lock (mu_mailbox_t mbox)
   mu_locker_modify (lock, &hints);
   if ((status = mu_locker_lock (lock)))
     {
-      mu_diag_output (MU_DIAG_NOTICE, _("locking mailbox `%s' failed: %s"),
-	      name ? name : "?", mu_strerror(status));
+      mu_diag_output (MU_DIAG_NOTICE,
+		      _("locking mailbox %s failed: %s"),
+		      name ? mu_quotename (name) : "?", mu_strerror (status));
       return status;
     }
   return 0;
