@@ -53,8 +53,8 @@ mu_tcpwrapper_access (int fd)
 }
 
 struct mu_cfg_param tcpwrapper_param[] = {
-  { "enable", mu_c_bool, &mu_tcp_wrapper_enable, 0, NULL,	      
-    N_("Enable TCP wrapper access control.  Default is \"yes\".") },	      
+  { "enable", mu_c_bool, &mu_tcp_wrapper_enable, 0, NULL,
+    N_("Enable TCP wrapper access control.  Default is 'yes'.") },
   { "daemon", mu_c_string, &mu_tcp_wrapper_daemon, 0, NULL,     
     N_("Set daemon name for TCP wrapper lookups.  Default is program name."), 
     N_("name") },							      
