@@ -1,5 +1,5 @@
 /* stdcapa.c -- Standard CLI capabilities for GNU Mailutils
-   Copyright (C) 2016-2023 Free Software Foundation, Inc.
+   Copyright (C) 2016-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -18,6 +18,7 @@
 # include <config.h>
 #endif
 #include <stdlib.h>
+#include <unistd.h>
 #include <mailutils/cli.h>
 #include <mailutils/cfg.h>
 #include <mailutils/nls.h>
@@ -31,6 +32,7 @@
 #include <mailutils/mu_auth.h>
 #include <mailutils/url.h>
 #include <mailutils/kwd.h>
+#include <mailutils/cstr.h>
 
 /* *************************************************************************
  * Logging section
@@ -40,7 +42,8 @@ cli_log_facility (struct mu_parseopt *po, struct mu_option *opt,
 		  char const *arg)
 {
   if (mu_string_to_syslog_facility (arg, &mu_log_facility))
-    mu_parseopt_error (po, _("unknown syslog facility `%s'"), arg);
+    mu_parseopt_error (po, _("unknown syslog facility: %s"),
+		       mu_quotename (arg));
 }
 
 static int
@@ -51,7 +54,8 @@ cb_facility (void *data, mu_config_value_t *val)
 
   if (mu_string_to_syslog_facility (val->v.string, &mu_log_facility))
     {
-      mu_error (_("unknown syslog facility `%s'"), val->v.string);
+      mu_error (_("unknown syslog facility: %s"),
+		mu_quotename (val->v.string));
       return 1;
     }
    return 0;
@@ -66,7 +70,7 @@ cb_severity (void *data, mu_config_value_t *val)
     return 1;
   if (mu_severity_from_string (val->v.string, &n))
     {
-      mu_error (_("unknown severity `%s'"), val->v.string);
+      mu_error (_("unknown severity: %s"), mu_quotename (val->v.string));
       return 1;
     }
   mu_log_severity_threshold = n;
@@ -119,8 +123,8 @@ cli_mailer (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
   int rc = mu_mailer_set_url_default (arg);
   if (rc != 0)
-    mu_parseopt_error (po, _("invalid mailer URL `%s': %s"),
-		       arg, mu_strerror (rc));
+    mu_parseopt_error (po, _("invalid mailer URL %s: %s"),
+		       mu_quotename (arg), mu_strerror (rc));
 }
 
 static struct mu_option mailer_option[] = {
@@ -208,8 +212,8 @@ cb_mail_spool (void *data, mu_config_value_t *val)
     return 1;
   rc = mu_set_mail_directory (val->v.string);
   if (rc)
-    mu_error (_("cannot set mail directory name to `%s': %s"),
-	      val->v.string, mu_strerror (rc));
+    mu_error (_("cannot set mail directory name to %s: %s"),
+	      mu_quotename (val->v.string), mu_strerror (rc));
   return rc;
 }
 
@@ -222,8 +226,8 @@ cb_mailbox_pattern (void *data, mu_config_value_t *val)
 
   rc = mu_set_mailbox_pattern (val->v.string);
   if (rc)
-    mu_error (_("cannot set mailbox pattern to `%s': %s"),
-	      val->v.string, mu_strerror (rc));
+    mu_error (_("cannot set mailbox pattern to %s: %s"),
+	      mu_quotename (val->v.string), mu_strerror (rc));
   return rc;
 }
 
@@ -324,7 +328,7 @@ cb_locker_flags (void *data, mu_config_value_t *val)
       char const *kw;
       if (mu_kwd_xlat_tok (flag_tab, *s, &kw))
 	{
-	  mu_error (_("invalid lock flag `%c'"), *s);
+	  mu_error (_("invalid lock flag '%c'"), *s);
 	}
       else if (*s == 'P')
 	{
@@ -536,8 +540,8 @@ cb_email_addr (void *data, mu_config_value_t *val)
 
   rc = mu_set_user_email (val->v.string);
   if (rc)
-    mu_error (_("invalid email address `%s': %s"),
-	      val->v.string, mu_strerror (rc));
+    mu_error (_("invalid email address %s: %s"),
+	      mu_quotename (val->v.string), mu_strerror (rc));
   return 0;
 }
 
@@ -551,8 +555,8 @@ cb_email_domain (void *data, mu_config_value_t *val)
 
   rc = mu_set_user_email_domain (val->v.string);
   if (rc)
-    mu_error (_("invalid email domain `%s': %s"),
-	      val->v.string, mu_strerror (rc));
+    mu_error (_("invalid email domain %s: %s"),
+	      mu_quotename (val->v.string), mu_strerror (rc));
   return 0;
 }
 
@@ -639,12 +643,12 @@ static struct mu_cfg_param mu_auth_param[] = {
     /* FIXME: The description is incomplete. MU-list is also allowed as
        argument */
     N_("Set a list of modules for authentication. Modlist is a "
-       "colon-separated list of module names or a word `clear' to "
+       "colon-separated list of module names or a word 'clear' to "
        "clear the previously set up values."),
     N_("modlist") },
   { "authorization", mu_cfg_callback, NULL, 0, cb_authorization,
     N_("Set a list of modules for authorization. Modlist is a "
-       "colon-separated list of module names or a word `clear' to "
+       "colon-separated list of module names or a word 'clear' to "
        "clear the previously set up values."),
     N_("modlist") },
   { NULL }
@@ -667,6 +671,61 @@ mu_auth_section_parser
   return 0;
 }
 
+static int
+cb_file_mode (void *data, mu_config_value_t *val)
+{
+  char *p;
+  int n;
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  n = strtoul (val->v.string, &p, 8);
+  if (p[0] || (n & ~0777))
+    mu_error (_("invalid mode specification: %s"), val->v.string);
+  *(int*) data = n;
+  return 0;
+}
+
+int mu_homedir_create;
+int mu_homedir_mode = 0755;
+
+int
+mu_homedir_assert (char const *dir, uid_t uid, gid_t gid)
+{
+  int rc;
+
+  if (!dir)
+    rc = EINVAL;
+  else if (mu_homedir_create)
+    {
+      int u = umask (0);
+      if ((rc = mu_mkinterdir (dir, mu_homedir_mode, 0755)) == 0)
+	{
+	  if (chown (dir, uid, gid))
+	    rc = errno;
+	}
+      umask (u);
+    }
+  else
+    {
+      struct stat st;
+      if (stat (dir, &st) == 0)
+	rc = S_ISDIR (st.st_mode) ? 0 : ENOTDIR;
+      else
+	rc = errno;
+    }
+  return rc;
+}
+
+static struct mu_cfg_param mu_homedir_param[] = {
+  { "create", mu_c_bool, &mu_homedir_create, 0, NULL,
+    N_("Create user home directory if it doesn't exist."),
+    N_("arg") },
+  { "mode", mu_cfg_callback, &mu_homedir_mode, 0, cb_file_mode,
+    N_("File mode for creating user home directories."),
+    N_("mode: octal") },
+  { NULL }
+};
+
 /* ************************************************************************* *
  * Registry of standard mailutils' capabilities                              *
  * ************************************************************************* */
@@ -679,7 +738,7 @@ struct mu_cli_capa mu_cli_std_capa[] = {
   { "locking", NULL, locking_cfg, NULL, NULL },
   { "address", NULL, address_cfg, NULL, NULL },
   { "auth",    NULL, mu_auth_param, mu_auth_section_parser },
-
+  { "homedir", NULL, mu_homedir_param, NULL },
   { NULL }
 };
 
