@@ -239,7 +239,7 @@ filter_read (mu_stream_t stream, char *buf, size_t size, size_t *pret)
 		{
 		  /* FIXME: What filter? Need some id. */
 		  mu_debug (MU_DEBCAT_FILTER, MU_DEBUG_ERROR,
-			    (_("filter returned `again' too many times")));
+			    (_("filter returned 'again' too many times")));
 		  again = 0;
 		}
 	      break;
@@ -293,7 +293,7 @@ filter_read (mu_stream_t stream, char *buf, size_t size, size_t *pret)
 			{
 			  /* FIXME: What filter? Need some id. */
 			  mu_debug (MU_DEBCAT_FILTER, MU_DEBUG_ERROR,
-				    (_("filter returned `again' too many times")));
+				    (_("filter returned 'again' too many times")));
 			  stop = 1;
 			}
 		    }
@@ -378,7 +378,7 @@ filter_write_internal (mu_stream_t stream, enum mu_filter_command cmd,
 	    {
 	      /* FIXME: What filter? Need some id. */
 	      mu_debug (MU_DEBCAT_FILTER, MU_DEBUG_ERROR,
-			(_("filter returned `again' too many times")));
+			(_("filter returned 'again' too many times")));
 	      again = 0;
 	    }
 	  break;
