@@ -113,21 +113,21 @@ static struct mu_cfg_param mu_ldap_param[] = {
   { NULL }
 };
 
-int									      
+int
 mu_ldap_section_parser
-   (enum mu_cfg_section_stage stage, const mu_cfg_node_t *node,	      
+   (enum mu_cfg_section_stage stage, const mu_cfg_node_t *node,
     const char *section_label, void **section_data,
     void *call_data, mu_cfg_tree_t *tree)
-{									      
-  switch (stage)							      
-    {									      
+{
+  switch (stage)
+    {
     case mu_cfg_section_start:
       ldap_param.enable = 1;
       break;
     default:
       break;
-    }									      
-  return 0;								      
+    }
+  return 0;
 }
 
 
@@ -163,7 +163,7 @@ _mu_conn_setup (LDAP **pld)
   char *ldapuri = NULL;
   LDAP *ld = NULL;
   int protocol = LDAP_VERSION3; /* FIXME: must be configurable */
-  
+
   if (ldap_param.debug)
     {
       if (ber_set_option (NULL, LBER_OPT_DEBUG_LEVEL, &ldap_param.debug)
@@ -172,8 +172,7 @@ _mu_conn_setup (LDAP **pld)
 
       if (ldap_set_option (NULL, LDAP_OPT_DEBUG_LEVEL, &ldap_param.debug)
 	  != LDAP_OPT_SUCCESS )
-	mu_error (_("could not set LDAP_OPT_DEBUG_LEVEL %d"),
-		  ldap_param.debug);
+	mu_error (_("cannot set LDAP_OPT_DEBUG_LEVEL %d"), ldap_param.debug);
     }
 
   if (ldap_param.url)
@@ -185,12 +184,12 @@ _mu_conn_setup (LDAP **pld)
 		    ldap_param.url, rc);
 	  return 1;
 	}
-      
+
       for (ludp = &ludlist; *ludp; )
 	{
 	  LDAPURLDesc *lud = *ludp;
 	  char **tmp;
-	  
+
 	  if (lud->lud_dn && lud->lud_dn[0]
 	      && (lud->lud_host == NULL || lud->lud_host[0] == '\0'))
 	    {
@@ -199,39 +198,40 @@ _mu_conn_setup (LDAP **pld)
 	      char *domain = NULL, *hostlist = NULL;
 	      size_t i;
 	      struct mu_wordsplit ws;
-	      
+
 	      if (ldap_dn2domain (lud->lud_dn, &domain) || !domain)
 		{
-		  mu_error (_("DNS SRV: cannot convert DN=\"%s\" into a domain"),
-			    lud->lud_dn );
+		  mu_error (_("DNS SRV: cannot convert DN=%s into a domain"),
+			    mu_quotename (lud->lud_dn));
 		  goto dnssrv_free;
 		}
-	      
+
 	      rc = ldap_domain2hostlist (domain, &hostlist);
 	      if (rc)
 		{
 		  mu_error (_("DNS SRV: cannot convert domain=%s into a hostlist"),
-			    domain);
+			    mu_quotename (domain));
 		  goto dnssrv_free;
 		}
 
 	      if (mu_wordsplit (hostlist, &ws, MU_WRDSF_DEFFLAGS))
 		{
-		  mu_error (_("DNS SRV: could not parse hostlist=\"%s\": %s"),
-			    hostlist, mu_wordsplit_strerror (&ws));
+		  mu_error (_("DNS SRV: could not parse hostlist=%s: %s"),
+			    mu_quotename (hostlist),
+			    mu_wordsplit_strerror (&ws));
 		  goto dnssrv_free;
 		}
-	      
+
 	      tmp = realloc (urls, sizeof(char *) * (nurls + ws.ws_wordc + 1));
 	      if (!tmp)
 		{
 		  mu_error ("DNS SRV %s", mu_strerror (errno));
 		  goto dnssrv_free;
 		}
-	      
+
 	      urls = tmp;
 	      urls[nurls] = NULL;
-	      
+
 	      for (i = 0; i < ws.ws_wordc; i++)
 		{
 		  urls[nurls + i + 1] = NULL;
@@ -244,9 +244,9 @@ _mu_conn_setup (LDAP **pld)
 		      goto dnssrv_free;
 		    }
 		}
-	      
+
 	      nurls += i;
-	      
+
 	    dnssrv_free:
 	      mu_wordsplit_free (&ws);
 	      ber_memfree (hostlist);
@@ -262,7 +262,7 @@ _mu_conn_setup (LDAP **pld)
 		}
 	      urls = tmp;
 	      urls[nurls + 1] = NULL;
-	      
+
 	      urls[nurls] = ldap_url_desc2str (lud);
 	      if (!urls[nurls])
 		{
@@ -271,9 +271,9 @@ _mu_conn_setup (LDAP **pld)
 		}
 	      nurls++;
 	    }
-	  
+
 	  *ludp = lud->lud_next;
-	  
+
 	  lud->lud_next = NULL;
 	  ldap_free_urldesc (lud);
 	}
@@ -285,14 +285,14 @@ _mu_conn_setup (LDAP **pld)
 	}
       else if (!urls)
 	return 1;
-      
+
       rc = mu_argcv_string (nurls, urls, &ldapuri);
       if (rc)
 	{
 	  mu_error ("%s", mu_strerror (rc));
 	  return 1;
 	}
-      
+
       ber_memvfree ((void **)urls);
     }
 
@@ -309,7 +309,7 @@ _mu_conn_setup (LDAP **pld)
       return 1;
     }
   free (ldapuri);
-  
+
   ldap_set_option (ld, LDAP_OPT_PROTOCOL_VERSION, &protocol);
 
   if (ldap_param.tls)
@@ -321,13 +321,13 @@ _mu_conn_setup (LDAP **pld)
 	  ldap_get_option (ld,
 			   LDAP_OPT_DIAGNOSTIC_MESSAGE,
 			   (void*)&msg);
-	  
+
 	  mu_error (_("ldap_start_tls failed: %s"), ldap_err2string (rc));
 	  mu_error (_("TLS diagnostics: %s"), msg);
 	  ldap_memfree (msg);
 
 	  ldap_unbind_ext (ld, NULL, NULL);
-	  
+
 	  return 1;
 	}
     }
@@ -389,7 +389,7 @@ _mu_ldap_bind (LDAP *ld)
       /* FIXME: Use debug output for that */
       mu_error ("ldap_bind: %s (%d)%s", ldap_err2string (err), err, msgbuf);
 
-      if (matched && *matched) 
+      if (matched && *matched)
 	mu_error ("matched DN: %s", matched);
 
       if (info && *info)
@@ -399,7 +399,7 @@ _mu_ldap_bind (LDAP *ld)
 	{
 	  int i;
 	  mu_error ("referrals:");
-	  for (i = 0; refs[i]; i++) 
+	  for (i = 0; refs[i]; i++)
 	    mu_error ("%s", refs[i]);
 	}
 
@@ -425,36 +425,33 @@ _mu_ldap_unbind (LDAP *ld)
     }
 }
 
-static int 
+static int
 _construct_attr_array (size_t *pargc, char ***pargv)
 {
-  size_t count, i;
-  char **argv;
+  size_t count, i, j;
+  char const **argv;
   mu_iterator_t itr = NULL;
-  
+
   mu_assoc_count (ldap_param.field_map, &count);
   if (count == 0)
     return MU_ERR_FAILURE;
   argv = calloc (count + 1, sizeof argv[0]);
 
   mu_assoc_get_iterator (ldap_param.field_map, &itr);
-  for (i = 0, mu_iterator_first (itr); !mu_iterator_is_done (itr);
+  for (i = j = 0, mu_iterator_first (itr); !mu_iterator_is_done (itr);
        mu_iterator_next (itr), i++)
     {
-      char *str;
-      mu_iterator_current (itr, (void**) &str); 
-      if ((argv[i] = strdup (str)) == NULL)
-        {
-          mu_argcv_free (i, argv);
-          return ENOMEM;
-        }
+      char const *key, *str;
+      mu_iterator_current_kv (itr, (const void **)&key, (void**) &str);
+      if (mu_isalpha (*str))
+	argv[j++] = str;
     }
   mu_iterator_destroy (&itr);
-  argv[i] = NULL;
-  
-  *pargc = count;
-  *pargv = argv;
-  
+  argv[j] = NULL;
+
+  *pargc = j;
+  *pargv = (char**) argv;
+
   return 0;
 }
 
@@ -469,7 +466,7 @@ get_quota (mu_off_t *pquota, const char *str)
     case 0:
       *pquota = n;
       break;
-      
+
     case ERANGE:
       mu_error (_("quota value is out of allowed range: %s"), str);
       break;
@@ -483,61 +480,43 @@ get_quota (mu_off_t *pquota, const char *str)
     }
 }
 
-static void
-_free_partial_auth_data (struct mu_auth_data *d)
-{
-  free (d->name);
-  free (d->passwd);
-  free (d->gecos);
-  free (d->dir);
-  free (d->shell);
-  free (d->mailbox);
-}
-
 static int
-_assign_partial_auth_data (struct mu_auth_data *d, const char *key,
-			   const char *val)
+_assign_partial_auth_data (char const *key, void *val, void *data)
 {
-  int rc = 0;
-  
+  struct mu_auth_data *d = data;
+
   if (strcmp (key, MU_AUTH_NAME) == 0)
-    rc = (d->name = strdup (val)) ? 0 : errno;
+    d->name = val;
   else if (strcmp (key, MU_AUTH_PASSWD) == 0)
-    rc = (d->passwd = strdup (val)) ? 0 : errno;
+    d->passwd = val;
   else if (strcmp (key, MU_AUTH_UID) == 0)
     d->uid = atoi (val);
   else if (strcmp (key, MU_AUTH_GID) == 0)
     d->gid = atoi (val);
   else if (strcmp (key, MU_AUTH_GECOS) == 0)
-    rc = (d->gecos = strdup (val)) ? 0 : errno;
+    d->gecos = val;
   else if (strcmp (key, MU_AUTH_DIR) == 0)
-    rc = (d->dir = strdup (val)) ? 0 : errno;
-  else if (strcmp (key, MU_AUTH_SHELL) == 0)   
-    rc = (d->shell = strdup (val)) ? 0 : errno;
+    d->dir = val;
+  else if (strcmp (key, MU_AUTH_SHELL) == 0)
+    d->shell = val;
   else if (strcmp (key, MU_AUTH_MAILBOX) == 0)
-    rc = (d->mailbox = strdup (val)) ? 0 : errno;
+    d->mailbox = val;
   else if (strcmp (key, MU_AUTH_QUOTA) == 0)
     get_quota (&d->quota, val);
-  return rc;
+  return 0;
 }
 
 static int
-_mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
-			struct mu_auth_data **return_data)
+fill_auth_data (LDAP *ld, LDAPMessage *msg, mu_assoc_t symtab,
+		struct mu_auth_data **return_data)
 {
   int rc;
-  BerElement *ber = NULL;
-  struct berval bv;
-  char *ufn = NULL;
   struct mu_auth_data d;
   mu_iterator_t itr = NULL;
-  
+  int need_pass2 = 0;
+
   memset (&d, 0, sizeof d);
-  
-  rc = ldap_get_dn_ber (ld, msg, &ber, &bv);
-  ufn = ldap_dn2ufn (bv.bv_val);
-  ldap_memfree (ufn);
-  
+
   mu_assoc_get_iterator (ldap_param.field_map, &itr);
   for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
        mu_iterator_next (itr))
@@ -545,27 +524,61 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
       char *key;
       char *attr;
       struct berval **values;
-      
+      char *s;
+
       mu_iterator_current_kv (itr, (const void **)&key, (void**) &attr);
-      values = ldap_get_values_len (ld, msg, attr);
-      if (!values || !values[0])
+
+      if (mu_isalpha (*attr))
 	{
-	  mu_error ("LDAP field `%s' (`%s') has NULL value",
-		    key, attr);
-	  _free_partial_auth_data (&d);
-	  return MU_ERR_READ;
+	  values = ldap_get_values_len (ld, msg, attr);
+	  if (!values || !values[0])
+	    {
+	      mu_error ("LDAP field `%s' (`%s') has NULL value",
+			key, attr);
+	      return MU_ERR_READ;
+	    }
+
+	  if ((s = strdup (values[0]->bv_val)) == NULL)
+	    return errno;
+
+	  rc = mu_assoc_install (symtab, key, s);
+
+	  ldap_value_free_len (values);
+	  if (rc)
+	    return rc;
 	}
-      
-      rc = _assign_partial_auth_data (&d, key, values[0]->bv_val);
-      
-      ldap_value_free_len (values);
-      if (rc)
+      else
+	need_pass2++;
+    }
+
+  if (need_pass2)
+    {
+      for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+	   mu_iterator_next (itr))
 	{
-	  _free_partial_auth_data (&d);
-	  return rc;
+	  char *key;
+	  char *attr;
+
+	  mu_iterator_current_kv (itr, (const void **)&key, (void**) &attr);
+
+	  if (!mu_isalpha (*attr))
+	    {
+	      char *val;
+
+	      if ((rc = mu_str_expand (&val, attr, symtab)) != 0)
+		return rc;
+	      if ((rc = mu_assoc_install (symtab, key, val)) != 0)
+		{
+		  free (val);
+		  return rc;
+		}
+	    }
 	}
     }
-  
+  mu_iterator_destroy (&itr);
+
+  mu_assoc_foreach (symtab, _assign_partial_auth_data, &d);
+
   rc = mu_auth_data_alloc (return_data,
 			   d.name,
 			   d.passwd,
@@ -579,11 +592,37 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
   if (rc == 0)
     mu_auth_data_set_quota (*return_data, d.quota);
 
-  _free_partial_auth_data (&d);
-  
   return rc;
 }
-    
+
+static int
+_mu_ldap_entry_to_auth_data (LDAP *ld, LDAPMessage *msg, char const *user,
+			     struct mu_auth_data **return_data)
+{
+  mu_assoc_t symtab;
+  int rc;
+  char *s;
+
+  if ((rc = mu_assoc_create (&symtab, 0)) != 0)
+    return rc;
+  mu_assoc_set_destroy_item (symtab, mu_list_free_item);
+  if ((s = strdup (user)) == NULL)
+    {
+      mu_assoc_destroy (&symtab);
+      return ENOMEM;
+    }
+  if ((rc = mu_assoc_install (symtab, "user", s)) != 0)
+    {
+      mu_assoc_destroy (&symtab);
+      free (s);
+      return rc;
+    }
+
+  rc = fill_auth_data (ld, msg, symtab, return_data);
+
+  mu_assoc_destroy (&symtab);
+  return rc;
+}
 
 static int
 _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
@@ -595,7 +634,7 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
   LDAPMessage *res, *msg;
   ber_int_t msgid;
   char *filter_str;
-  
+
   rc = _construct_attr_array (&nattrs, &attrs);
   if (rc)
     return rc;
@@ -603,26 +642,26 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
   rc = mu_str_vexpand (&filter_str, filter_pat, "user", key, NULL);
   if (rc)
     {
-      mu_argcv_free (nattrs, attrs);
+      free (attrs);
       if (rc == MU_ERR_FAILURE)
 	{
-	  mu_error (_("cannot expand line `%s': %s"), filter_pat,
+	  mu_error (_("cannot expand line %s: %s"), mu_quotename (filter_pat),
 		    filter_str);
 	  free (filter_str);
 	}
       else
 	{
-	  mu_error (_("cannot expand line `%s': %s"), filter_pat,
+	  mu_error (_("cannot expand line %s: %s"), mu_quotename (filter_pat),
 		    mu_strerror (rc));
 	}
       return rc;
     }
-  
+
   rc = ldap_search_ext (ld, ldap_param.base, LDAP_SCOPE_SUBTREE,
 			filter_str, attrs, 0,
 			NULL, NULL, NULL, -1, &msgid);
   free (filter_str);
-  mu_argcv_free (nattrs, attrs);
+  free (attrs);
 
   if (rc != LDAP_SUCCESS)
     {
@@ -648,12 +687,11 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
     }
   if (rc > 1)
     mu_error ("LDAP: too many entries for key %s", key);
-      
 
   msg = ldap_first_entry (ld, res);
-  rc = _mu_entry_to_auth_data (ld, msg, return_data);
+  rc = _mu_ldap_entry_to_auth_data (ld, msg, key, return_data);
   ldap_msgfree (res);
-  
+
   return rc;
 }
 
@@ -666,7 +704,7 @@ static int
 chk_crypt (const char *db_pass, const char *pass)
 {
   return strcmp (db_pass, crypt (pass, db_pass)) == 0 ?
-              0 : MU_ERR_AUTH_FAILURE;
+	      0 : MU_ERR_AUTH_FAILURE;
 }
 
 static int
@@ -676,7 +714,7 @@ chk_md5 (const char *db_pass, const char *pass)
   unsigned char d1[16];
   struct mu_md5_ctx md5context;
   mu_stream_t str = NULL, flt = NULL;
-  
+
   mu_md5_init_ctx (&md5context);
   mu_md5_process_bytes (pass, strlen (pass), &md5context);
   mu_md5_finish_ctx (&md5context, md5digest);
@@ -687,9 +725,9 @@ chk_md5 (const char *db_pass, const char *pass)
 
   mu_stream_read (flt, (char*) d1, sizeof d1, NULL);
   mu_stream_destroy (&flt);
-  
+
   return memcmp (md5digest, d1, sizeof md5digest) == 0 ?
-                  0 : MU_ERR_AUTH_FAILURE;
+		  0 : MU_ERR_AUTH_FAILURE;
 }
 
 static int
@@ -701,7 +739,7 @@ chk_smd5 (const char *db_pass, const char *pass)
   struct mu_md5_ctx md5context;
   mu_stream_t str = NULL, flt = NULL;
   size_t size;
-  
+
   size = strlen (db_pass);
   mu_static_memory_stream_create (&str, db_pass, size);
   mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
@@ -713,7 +751,7 @@ chk_smd5 (const char *db_pass, const char *pass)
       mu_stream_destroy (&flt);
       return ENOMEM;
     }
-  
+
   mu_stream_read (flt, (char*) d1, size, &size);
   mu_stream_destroy (&flt);
 
@@ -722,14 +760,14 @@ chk_smd5 (const char *db_pass, const char *pass)
       mu_error ("malformed SMD5 password: %s", db_pass);
       return MU_ERR_FAILURE;
     }
-  
+
   mu_md5_init_ctx (&md5context);
   mu_md5_process_bytes (pass, strlen (pass), &md5context);
   mu_md5_process_bytes (d1 + 16, size - 16, &md5context);
   mu_md5_finish_ctx (&md5context, md5digest);
 
   rc = memcmp (md5digest, d1, sizeof md5digest) == 0 ?
-                  0 : MU_ERR_AUTH_FAILURE;
+		  0 : MU_ERR_AUTH_FAILURE;
   free (d1);
   return rc;
 }
@@ -741,7 +779,7 @@ chk_sha (const char *db_pass, const char *pass)
   unsigned char d1[20];
   mu_stream_t str = NULL, flt = NULL;
   struct mu_sha1_ctx sha1context;
-   
+
   mu_sha1_init_ctx (&sha1context);
   mu_sha1_process_bytes (pass, strlen (pass), &sha1context);
   mu_sha1_finish_ctx (&sha1context, sha1digest);
@@ -752,9 +790,9 @@ chk_sha (const char *db_pass, const char *pass)
 
   mu_stream_read (flt, (char*) d1, sizeof d1, NULL);
   mu_stream_destroy (&flt);
-  
+
   return memcmp (sha1digest, d1, sizeof sha1digest) == 0 ?
-                  0 : MU_ERR_AUTH_FAILURE;
+		  0 : MU_ERR_AUTH_FAILURE;
 }
 
 static int
@@ -766,7 +804,7 @@ chk_ssha (const char *db_pass, const char *pass)
   struct mu_sha1_ctx sha1context;
   mu_stream_t str = NULL, flt = NULL;
   size_t size;
-  
+
   size = strlen (db_pass);
   mu_static_memory_stream_create (&str, db_pass, size);
   mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
@@ -778,7 +816,7 @@ chk_ssha (const char *db_pass, const char *pass)
       mu_stream_destroy (&flt);
       return ENOMEM;
     }
-  
+
   mu_stream_read (flt, (char*) d1, size, &size);
   mu_stream_destroy (&flt);
 
@@ -787,14 +825,14 @@ chk_ssha (const char *db_pass, const char *pass)
       mu_error ("malformed SSHA1 password: %s", db_pass);
       return MU_ERR_FAILURE;
     }
-  
+
   mu_sha1_init_ctx (&sha1context);
   mu_sha1_process_bytes (pass, strlen (pass), &sha1context);
   mu_sha1_process_bytes (d1 + 20, size - 20, &sha1context);
   mu_sha1_finish_ctx (&sha1context, sha1digest);
 
   rc = memcmp (sha1digest, d1, sizeof sha1digest) == 0 ?
-                  0 : MU_ERR_AUTH_FAILURE;
+		  0 : MU_ERR_AUTH_FAILURE;
   free (d1);
   return rc;
 }
@@ -843,7 +881,7 @@ mu_ldap_authenticate (struct mu_auth_data **return_data MU_ARG_UNUSED,
       char *algo;
       pwcheck_fp pwcheck;
 
-      
+
       algo = db_pass + 1;
       for (len = 0; algo[len] != '}'; len++)
 	if (algo[len] == 0)
@@ -862,7 +900,7 @@ mu_ldap_authenticate (struct mu_auth_data **return_data MU_ARG_UNUSED,
 	  return MU_ERR_FAILURE;
 	}
     }
-  
+
   return strcmp (db_pass, pass) == 0 ? 0 : MU_ERR_AUTH_FAILURE;
 }
 
@@ -914,7 +952,7 @@ mu_auth_ldap_user_by_uid (struct mu_auth_data **return_data,
 #else
 # define module_init NULL
 # define mu_ldap_authenticate mu_auth_nosupport
-# define mu_auth_ldap_user_by_name mu_auth_nosupport 
+# define mu_auth_ldap_user_by_name mu_auth_nosupport
 # define mu_auth_ldap_user_by_uid mu_auth_nosupport
 # define mu_ldap_param NULL
 
