@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005-2024 Free Software Foundation, Inc.
+   Copyright (C) 2005-2025 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -211,10 +211,11 @@ regex_comparator (void *item, void *data)
   if (regcomp (&preg, item,
 	       REG_EXTENDED | REG_NOSUB | REG_NEWLINE | REG_ICASE))
     {
+      // FIXME: detailed error?
       mu_sieve_error (d->mach,
-		      _("%lu: cannot compile regular expression \"%s\""),
+		      _("%lu: cannot compile regular expression %s"),
 		      (unsigned long) mu_sieve_get_message_num (d->mach),
-		      (char*) item);
+		      mu_quotename ((char*) item));
       return 0;
     }
   rc = regexec (&preg, d->email, 0, NULL, 0) == 0;
@@ -579,9 +580,9 @@ add_header (void *item, void *data)
   if (rc)
     {
       mu_sieve_error (hc->mach,
-		      _("%lu: can't add header \"%s\": %s"),
+		      _("%lu: cannot add header %s: %s"),
 		      (unsigned long) mu_sieve_get_message_num (hc->mach),
-		      str, mu_strerror (rc));
+		      mu_quotename (str), mu_strerror (rc));
       return 0;
     }
 
@@ -591,7 +592,7 @@ add_header (void *item, void *data)
   
   if (rc)
     mu_sieve_error (hc->mach,
-		    _("%lu: can't add header \"%s\": %s"),
+		    _("%lu: cannot add header \"%s\": %s"),
 		    (unsigned long) mu_sieve_get_message_num (hc->mach),
 		    str, mu_strerror (rc));
   return 0;
