@@ -120,7 +120,7 @@ list_create_or_die (void)
   status = mu_list_create (&list);
   if (status)
     {
-      mu_error (_("can't create list: %s"), mu_strerror (status));
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, status);
       exit (1);
     }
   return list;
@@ -1809,8 +1809,8 @@ mh_alias_get_address (const char *name, mu_address_t *paddr, int *incl)
 	  mu_iterator_current (itr, (void **)&item);
 	  if (mu_address_create (&a, item))
 	    {
-	      mu_error (_("Error expanding aliases -- invalid address `%s'"),
-			item);
+	      mu_error (_("Error expanding aliases -- invalid address %s"),
+			mu_quotename (item));
 	    }
 	  else
 	    {
@@ -1949,7 +1949,7 @@ mh_read_aliases (void)
 
       if (mu_wordsplit (p, &ws, MU_WRDSF_DEFFLAGS))
 	{
-	  mu_error (_("cannot split line `%s': %s"), p,
+	  mu_error (_("cannot split line %s: %s"), mu_quotename (p),
 		    mu_wordsplit_strerror (&ws));
 	}
       else
