@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007-2024 Free Software Foundation, Inc.
+   Copyright (C) 2007-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,16 +43,16 @@ open_bulletin_mailbox (mu_mailbox_t *pmbox)
   
   if ((status = mu_mailbox_create (&tmbox, bulletin_mbox_name)) != 0)
     {
-      mu_error (_("cannot create bulletin mailbox `%s': %s"),
-		bulletin_mbox_name, mu_strerror (status));
+      mu_error (_("cannot create bulletin mailbox %s: %s"),
+		mu_quotename (bulletin_mbox_name), mu_strerror (status));
       return 1;
     }
 
   if ((status = mu_mailbox_open (tmbox, MU_STREAM_READ)) != 0)
     {
       mu_mailbox_destroy (&tmbox);
-      mu_error (_("cannot open bulletin mailbox `%s': %s"),
-		bulletin_mbox_name, mu_strerror (status));
+      mu_error (_("cannot open bulletin mailbox %s: %s"),
+		mu_quotename (bulletin_mbox_name), mu_strerror (status));
       return 1;
     }
   if (!pmbox)
@@ -107,7 +107,7 @@ write_popbull_file (size_t num)
   fp = fopen (filename, "w");
   if (fp)
     {
-      fprintf (fp, "%s\n", mu_umaxtostr (0, num));
+      fprintf (fp, "%s\n", mu_umaxtostr (num));
       fclose (fp);
       rc = 0;
     }
@@ -204,8 +204,8 @@ read_bulletin_db (size_t *pnum)
     rc = 0;
   else
     {
-      mu_error (_("wrong bulletin database format for `%s'"),
-		username);
+      mu_error (_("wrong bulletin database format for %s"),
+		mu_quotename (username));
     }
   
   free (buf);
@@ -251,7 +251,7 @@ write_bulletin_db (size_t num)
 
   key.mu_dptr = username;
   key.mu_dsize = strlen (username);
-  p = mu_umaxtostr (0, num);
+  p = mu_umaxtostr (num);
   data.mu_dptr = (char *) p;
   data.mu_dsize = strlen (p);
 
