@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2024 Free Software Foundation, Inc.
+   Copyright (C) 1999-2025 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -54,7 +54,9 @@ pop3d_stat (char *arg, struct pop3d_session *sess)
 	  num++;
 	}
     }
-  pop3d_outf ("+OK %s %s\n", mu_umaxtostr (0, num), mu_umaxtostr (1, tsize));
+  pop3d_outf ("+OK %s %s\n", 
+              mu_umaxtostr_n (0, num), 
+              mu_umaxtostr_n (1, tsize));
 
   return OK;
 }
